/*
 *  generator of eagle firmware image
 *
 *  Copyright (c) 2004 by Castet Matthieu <castet.matthieu@free.fr>
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */

#include <inttypes.h>
#include "eu_firmware.h"
#include "elsa_firmware/FirmwareUsbElsa.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <asm/byteorder.h>

/* crc stuff shamelessly stolen from linux-2.6.0/lib/crc32.c */

/*
 * There are multiple 16-bit CRC polynomials in common use, but this is
 * *the* standard CRC-32 polynomial, first popularized by Ethernet.
 * x^32+x^26+x^23+x^22+x^16+x^12+x^11+x^10+x^8+x^7+x^5+x^4+x^2+x^1+x^0
 */
#define CRCPOLY_BE 0x04c11db7

uint32_t crc32_be(uint32_t crc, unsigned char const *p, size_t len)
{
	int i;
	while (len--) {
		crc ^= *p++ << 24;
		for (i = 0; i < 8; i++)
			crc = (crc << 1) ^ ((crc & 0x80000000) ? CRCPOLY_BE : 0);
	}
	return crc;
}

static void mk_fir(char * name, eu_hex_record_t *pfirm)
{
	int fd;
	uint32_t crc = 0;
	fd = creat(name, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
	if (fd == -1) {
		perror("");
		return ;
	}
	write(fd, &crc, 4); /*for crc*/
	while(pfirm->type == 0) {
		uint16_t add = __cpu_to_le16(pfirm->address);
		write(fd, &pfirm->length, sizeof(pfirm->length)/*1*/);
		crc = crc32_be(crc, &pfirm->length, sizeof(pfirm->length)/*1*/);
		write(fd, &add, sizeof(pfirm->address)/*2*/);
		crc = crc32_be(crc, (uint8_t*)&add, sizeof(pfirm->address)/*2*/);
		write(fd, pfirm->data, pfirm->length);
		crc = crc32_be(crc, pfirm->data, pfirm->length);
		pfirm++;
	}
	crc = __cpu_to_le32(crc);
	lseek(fd, 0, SEEK_SET);
	write(fd, &crc, 4); /*for crc*/
	close(fd);
}

int main(int argc, char **argv) {
	if (argc == 2)
		chdir(argv[1]);
		
	mk_fir("eagleI.fw", eu_eagle_I_firmware);
	mk_fir("eagleII.fw", eu_eagle_II_firmware);
	mk_fir("eagleIII.fw", eu_eagle_III_firmware);
	mk_fir("adi930.fw", Firmware8051_Eagle);
	return 0;
}
